from dataclasses import dataclass
from enum import Enum


class Proximity(str, Enum):
    VICINITY = "VC"


class Intensity(str, Enum):
    LIGHT = "-"
    HEAVY = "+"
    MODERATE = ""


class Descriptor(str, Enum):
    PATCHES = "BC"
    BLOWING = "BL"
    LOW_DRIFTING = "DR"
    FREEZING = "FZ"
    SHALLOW = "MI"
    PARTIAL = "PR"
    SHOWER = "SH"
    THUNDERSTORM = "TS"


class Precipitation(str, Enum):
    DRIZZLE = "DZ"
    HAIL = "GR"
    SMALL_HAIL = "GS"
    ICE_CRYSTALS = "IC"
    ICE_PELLETS = "PL"
    RAIN = "RA"
    SNOW_GRAINS = "SG"
    SNOW = "SN"
    UNKNOWN_PRECIPITATION = "UP"


class Obscuration(str, Enum):
    MIST = "BR"
    WIDESPREAD_DUST = "DU"
    FOG = "FG"
    SMOKE = "FU"
    HAZE = "HZ"
    SAND = "SA"
    VOLCANIC_ASH = "VA"


class Other(str, Enum):
    DUSTSTORM = "DS"
    FUNNEL_CLOUD = "FC"
    SAND_WHIRLS = "PO"
    SQUALLS = "SQ"
    SANDSTORM = "SS"


@dataclass
class Wx:
    proximity: Proximity | None = None
    intensity: Intensity | None = None
    descriptor: Descriptor | None = None
    precipitation: Precipitation | None = None
    obscuration: Obscuration | None = None
    other: Other | None = None

    def taf_encode(self):
        parts = []
        if self.proximity:
            parts.append(self.proximity.value)
        if self.intensity:
            parts.append(self.intensity.value)
        if self.descriptor:
            parts.append(self.descriptor.value)
        if self.precipitation:
            parts.append(self.precipitation.value)
        if self.obscuration:
            parts.append(self.obscuration.value)
        if self.other:
            parts.append(self.other.value)
        return "".join(parts)


def parse(token: str) -> Wx | None:
    index = 0

    # Proximity
    proximity = None
    for key in Proximity:
        if token[index:].startswith(key):
            proximity = key
            index += len(key)
            break

    # Intensity
    intensity = None
    for key in Intensity:
        if token[index:].startswith(key):
            intensity = key
            index += len(key)
            break

    # Descriptor
    descriptor = None
    for key in Descriptor:
        if token[index:].startswith(key):
            descriptor = key
            index += len(key)
            break

    # Precipitation
    precipitation = None
    for key in Precipitation:
        if token[index:].startswith(key):
            precipitation = key
            index += len(key)
            break

    # Obscuration
    obscuration = None
    for key in Obscuration:
        if token[index:].startswith(key):
            obscuration = key
            index += len(key)
            break

    # Other
    other = None
    for key in Other:
        if token[index:].startswith(key):
            other = key
            index += len(key)
            break

    if (precipitation is not None) or (obscuration is not None) or (other is not None):
        return Wx(
            proximity=proximity,
            intensity=intensity,
            descriptor=descriptor,
            precipitation=precipitation,
            obscuration=obscuration,
            other=other,
        )
    else:
        return None


decode = parse



import re
from dataclasses import dataclass, field
from enum import Enum
from collections import namedtuple
from detaf import wx, temperature
from detaf.cloud import Cloud
from detaf.temperature import Temperature
from detaf import wx as weather

__all__ = [
    "Change",
    "Cloud",
    "decode",
    "encode",
    "From",
    "issue",
    "Modification",
    "NSW",
    "period",
    "TAF",
    "temperature",
    "Temperature",
    "Visibility",
    "weather",
    "WeatherCondition",
    "Wind",
]


class Change(str, Enum):
    BECMG = "BECMG"
    TEMPO = "TEMPO"


@dataclass
class From:
    """Forecast condition change after certain time"""

    day: int
    hour: int
    minute: int

    @classmethod
    def taf_decode(cls, text: str):
        if text[:2] == "FM":
            day = int(text[2:4])
            hour = int(text[4:6])
            minute = int(text[6:8])
            return cls(day, hour, minute)

    def taf_encode(self) -> str:
        return f"FM{self.day:02}{self.hour:02}{self.minute:02}"


class Modification(str, Enum):
    AMMENDED = "AMD"
    CORRECTED = "COR"


issue = namedtuple("issue", "day hour minute")
period = namedtuple("period", "begin end")
dayhour = namedtuple("dayhour", "day hour")


@dataclass
class Visibility:
    distance: int

    def taf_encode(self):
        return f"{self.distance}"

    @staticmethod
    def taf_decode(token: str):
        pattern = re.compile(r"[0-9]{4}")
        if len(token) == 4 and pattern.match(token):
            return Visibility(int(token))
        else:
            return None


@dataclass
class Wind:
    direction: int
    speed: int
    gust: int | None = None

    def taf_encode(self):
        if self.gust:
            return f"{self.direction:03}{self.speed:02}G{self.gust:02}KT"
        else:
            return f"{self.direction:03}{self.speed:02}KT"

    @staticmethod
    def taf_decode(token: str):
        if token.endswith("KT"):
            if "G" in token:
                gust = int(token[6:8])
            else:
                gust = None
            direction = None
            try:
                direction = int(token[:3])
            except ValueError:
                direction = token[:3]
                assert direction == "VRB", "must be either VRB or 3-digit number"
            return Wind(direction, int(token[3:5]), gust)
        else:
            return None  # Explicit is better than implicit


class NSW(str, Enum):
    NO_SIGNIFICANT_WEATHER = "NSW"

    def taf_encode(self):
        return "NSW"

    @staticmethod
    def taf_decode(token: str):
        if token == NSW.NO_SIGNIFICANT_WEATHER:
            return NSW.NO_SIGNIFICANT_WEATHER
        else:
            return None


Phenomenon = Visibility | Wind | Cloud


@dataclass
class WeatherCondition:
    period: period = None
    probability: int | None = None
    change: Change | None = None
    fm: From | None = None
    phenomena: list[Phenomenon] = field(default_factory=list)

    def taf_encode(self):
        parts = []
        if self.probability:
            parts.append(f"PROB{self.probability:02}")
        if self.change:
            parts.append(self.change.value)
        if self.period:
            parts.append(encode_period(self.period))
        for phenomenon in self.phenomena:
            parts.append(phenomenon.taf_encode())
        return " ".join(parts)


class Format(str, Enum):
    TAF = "TAF"
    # METAR = "METAR"
    # SPECI = "SPECI"


@dataclass
class TAF:
    format: Format | None = Format.TAF
    modification: Modification | None = None
    icao_identifier: str | None = None
    issue_time: str = None
    weather_conditions: list[WeatherCondition] = field(default_factory=list)

    def __iter__(self):
        if self.format:
            yield self.format
        if self.modification:
            yield self.modification
        if self.icao_identifier:
            yield self.icao_identifier
        if self.issue_time:
            yield self.issue_time
        yield from self.weather_conditions

    def taf_encode(self) -> str:
        return " ".join(encode(item) for item in self)


def decode(bulletin: str) -> TAF:
    words = bulletin.strip().split()
    words = [word.strip() for word in words if word != ""]

    # Station information and bulletin time
    format, cursor = parse_format(words, 0)
    modification, cursor = parse_modification(words, cursor)
    icao_identifier, cursor = parse_icao_identifier(words, cursor)
    issue_time, cursor = parse_issue_time(words, cursor)

    # Weather condition(s)
    conditions = []
    while cursor < len(words):
        condition, cursor = parse_condition(words, cursor)
        if condition:
            conditions.append(condition)
        else:
            print(f"unrecognised token: {words[cursor]}")
            cursor += 1  # Skip: bad token

    return TAF(format, modification, icao_identifier, issue_time, conditions)


def parse_format(tokens, cursor=0):
    token = peek(tokens, cursor)
    for key in Format:
        if token == key:
            return key, cursor + 1
    return None, cursor


def parse_modification(tokens, cursor=0) -> (Modification | None, int):
    token = peek(tokens, cursor)
    for key in Modification:
        if token == key:
            return key, cursor + 1
    return None, cursor


def parse_icao_identifier(tokens, cursor=0):
    icao_identifier = peek(tokens, cursor)
    if icao_identifier:
        return icao_identifier, cursor + 1
    else:
        return None, cursor


def parse_condition(tokens, cursor=0):
    probability, cursor = parse_probability(tokens, cursor)
    fm, cursor = parse_decoder(From.taf_decode)(tokens, cursor)
    if fm:
        phenomena, cursor = parse_phenomena(tokens, cursor)
        return WeatherCondition(
            fm=fm, probability=probability, phenomena=phenomena
        ), cursor
    else:
        change, cursor = parse_change(tokens, cursor)
        period, cursor = parse_period(tokens, cursor)
        phenomena, cursor = parse_phenomena(tokens, cursor)
        if period:
            return WeatherCondition(
                period, probability, change, phenomena=phenomena
            ), cursor
    return None, cursor


def parse_phenomena(tokens, cursor):
    phenomena = []
    while cursor < len(tokens):
        phenomenon, cursor = parse_phenomenon(tokens, cursor)
        if phenomenon:
            phenomena.append(phenomenon)
        else:
            break
    return phenomena, cursor


def parse_issue_time(tokens, cursor=0):
    token = peek(tokens, cursor)
    if not token:
        return None, cursor

    # Parse ddhhMMZ format into tuple
    day = int(token[:2])
    hour = int(token[2:4])
    minute = int(token[4:6])
    return issue(day, hour, minute), cursor + 1


def parse_period(tokens, cursor=0):
    token = peek(tokens, cursor)
    if not token:
        return None, cursor
    if not is_period(token):
        return None, cursor

    # Parse ddhh/ddhh format into period
    begin = dayhour(int(token[0:2]), int(token[2:4]))
    end = dayhour(int(token[5:7]), int(token[7:9]))
    return period(begin, end), cursor + 1


def is_period(token):
    return (len(token) == 9) and (token[4] == "/")


def parse_probability(tokens, cursor=0):
    token = peek(tokens, cursor)
    if token.startswith("PROB"):
        return int(token[4:6]), cursor + 1
    else:
        return None, cursor


def parse_change(tokens, cursor=0):
    token = peek(tokens, cursor)
    if token == "TEMPO":
        return Change.TEMPO, cursor + 1
    elif token == "BECMG":
        return Change.BECMG, cursor + 1
    else:
        return None, cursor


def parse_phenomenon(tokens, cursor=0):
    for parser in [
        parse_decoder(Visibility.taf_decode),
        parse_decoder(Wind.taf_decode),
        parse_decoder(Cloud.taf_decode),
        parse_decoder(NSW.taf_decode),
        parse_decoder(wx.parse),
        parse_decoder(Temperature.taf_decode),
    ]:
        phenomenon, cursor = parser(tokens, cursor)
        if phenomenon:
            return phenomenon, cursor
    return None, cursor


def parse_decoder(decoder):
    def parser(tokens, cursor=0):
        token = peek(tokens, cursor)
        obj = decoder(token)
        if obj:
            return obj, cursor + 1
        else:
            return None, cursor

    return parser


def peek(tokens, cursor):
    try:
        return tokens[cursor]
    except IndexError:
        return None


def encode(item) -> str:
    if isinstance(item, issue):
        return encode_issue_time(item)
    elif isinstance(item, period):
        return encode_period(item)
    elif isinstance(item, str):
        return item
    else:
        return item.taf_encode()


def encode_issue_time(value):
    return f"{value.day:02}{value.hour:02}{value.minute:02}Z"


def encode_period(value):
    return f"{value.begin.day:02}{value.begin.hour:02}/{value.end.day:02}{value.end.hour:02}"



import detaf
import pytest
from hypothesis import given
from hypothesis.strategies import text, sampled_from, integers, from_regex


# WEATHER


@given(
    intensity=sampled_from(detaf.weather.Intensity),
    descriptor=sampled_from(detaf.weather.Descriptor),
    precipitation=sampled_from(detaf.weather.Precipitation),
)
def test_decode_weather_given_intensity_descriptor_precipitation(
    intensity, descriptor, precipitation
):
    report = intensity + descriptor + precipitation
    assert detaf.weather.decode(report).intensity == intensity
    assert detaf.weather.decode(report).descriptor == descriptor
    assert detaf.weather.decode(report).precipitation == precipitation


@given(
    intensity=sampled_from(detaf.weather.Intensity),
    precipitation=sampled_from(detaf.weather.Precipitation),
)
def test_decode_weather_given_intensity_and_precipitation(intensity, precipitation):
    report = intensity + precipitation
    assert detaf.weather.decode(report).intensity == intensity
    assert detaf.weather.decode(report).precipitation == precipitation


@given(
    proximity=sampled_from(detaf.weather.Proximity),
    precipitation=sampled_from(detaf.weather.Precipitation),
)
def test_decode_weather_given_proximity_and_precipitation(proximity, precipitation):
    report = proximity + precipitation
    assert detaf.weather.decode(report).proximity == proximity
    assert detaf.weather.decode(report).precipitation == precipitation


@given(
    precipitation=sampled_from(detaf.weather.Precipitation),
)
def test_decode_weather_given_precipitation(precipitation):
    report = precipitation
    assert detaf.weather.decode(report).precipitation == precipitation


@given(
    obscuration=sampled_from(detaf.weather.Obscuration),
)
def test_decode_weather_given_obscuration(obscuration):
    report = obscuration
    assert detaf.weather.decode(report).obscuration == obscuration


@given(
    other=sampled_from(detaf.weather.Other),
)
def test_decode_weather_given_other(other):
    report = other
    assert detaf.weather.decode(report).other == other


# VISIBILITY


@given(distance=integers(min_value=0, max_value=9999))
def test_visibility(distance):
    report = f"TAF AAAA 000000Z 0000/0000 {distance:04}"
    assert detaf.decode(report).weather_conditions[0].phenomena[0].distance == distance


# WIND


@given(
    direction=integers(min_value=0, max_value=360),
    speed=integers(min_value=0, max_value=99),
)
def test_wind_direction_and_speed(direction, speed):
    report = f"TAF AAAA 000000Z 0000/0000 {direction:03}{speed:02}KT"
    wind = detaf.decode(report).weather_conditions[0].phenomena[0]
    assert wind.speed == speed
    assert wind.direction == direction


@given(
    direction=integers(min_value=0, max_value=360),
    speed=integers(min_value=0, max_value=99),
    gust=integers(min_value=0, max_value=99),
)
def test_wind_direction_speed_gust(direction, speed, gust):
    report = f"TAF AAAA 000000Z 0000/0000 {direction:03}{speed:02}G{gust:02}KT"
    wind = detaf.decode(report).weather_conditions[0].phenomena[0]
    assert wind.speed == speed
    assert wind.direction == direction
    assert wind.gust == gust


# Encode


@given(
    modification=sampled_from(detaf.Modification),
    icao_identifier=text("ABCDEFGHIJKLMNOPQRSTUVWXYZ", min_size=4, max_size=4),
)
def test_encode_decoded_report(modification, icao_identifier):
    report = f"TAF"
    report += f" {modification.value}"
    report += f" {icao_identifier}"
    report += " 000000Z 0000/0000 9999 PROB30 TEMPO 0000/0000 9999"
    assert detaf.encode(detaf.decode(report)) == report


# CLOUD
@pytest.mark.parametrize("report", ["BKN008", "CAVOK", "SCT018CB", "FEW030TCU"])
def test_encode_decode_cloud(report):
    assert detaf.encode(detaf.cloud.decode(report)) == report


# Temperature
@pytest.mark.parametrize("report", ["TX27/1815Z", "TN15/1806Z"])
def test_encode_decode_temperature(report):
    assert detaf.encode(detaf.temperature.decode(report)) == report


def test_encode_decode():
    assert detaf.encode(detaf.decode("")) == ""


# FMDDHHMM syntax


@given(
    day=integers(min_value=0, max_value=31),
    hour=integers(min_value=0, max_value=31),
    minute=integers(min_value=0, max_value=31),
)
def test_fm_syntax(day, hour, minute):
    text = f"FM{day:02}{hour:02}{minute:02}"
    assert detaf.From.taf_decode(text).day == day
    assert detaf.From.taf_decode(text).hour == hour
    assert detaf.From.taf_decode(text).minute == minute


def test_fm_syntax_given_invalid_text():
    assert detaf.From.taf_decode("XX000000") is None


@given(
    day=integers(min_value=0, max_value=31),
    hour=integers(min_value=0, max_value=31),
    minute=integers(min_value=0, max_value=31),
)
def test_fm_encode(day, hour, minute):
    assert (
        detaf.From(day, hour, minute).taf_encode() == f"FM{day:02}{hour:02}{minute:02}"
    )



from dataclasses import dataclass
from enum import Enum


class CloudDescription(str, Enum):
    NO_SIGNIFICANT_CLOUD = "NSC"
    CEILING_AND_VISIBILITY_OK = "CAVOK"
    FEW = "FEW"
    BROKEN = "BKN"
    OVERCAST = "OVC"
    SCATTERED = "SCT"
    SKY_CLEAR = "SKC"


class Type(str, Enum):
    CUMULONIMBUS = "CB"
    TOWERING_CUMULONIMBUS = "TCU"


@dataclass
class Cloud:
    description: CloudDescription
    height: int | None = None
    type: Type | None = None

    def taf_encode(self):
        parts = [self.description.value]
        if self.height:
            h = self.height // 100  # integer divide
            parts.append(f"{h:03}")
        if self.type:
            parts.append(self.type.value)
        return "".join(parts)

    @staticmethod
    def taf_decode(token: str):
        # Description
        pointer = 0
        description = None
        for key in CloudDescription:
            if token.startswith(key):
                description = key
                pointer += len(key)
                break

        # Height
        height = None
        try:
            width = 3
            height = 100 * int(token[pointer : pointer + width])
            pointer += width
        except ValueError:
            pass

        # Type
        type = None
        for key in Type:
            width = len(key)
            if token[pointer : pointer + width] == key:
                type = key
                pointer += width

        if description:
            return Cloud(description, height, type)
        else:
            return None


def decode(text):
    return Cloud.taf_decode(text)



from enum import Enum
from collections import namedtuple
from dataclasses import dataclass


class Limit(str, Enum):
    MIN = "N"
    MAX = "X"


dayhour = namedtuple("dayhour", "day hour")


@dataclass
class Temperature:
    value: int
    limit: Limit
    at: dayhour

    @staticmethod
    def taf_decode(token: str):
        if token.startswith("TX") or token.startswith("TN"):
            value = int(token[2:4])
            limit = token[1]
            at = dayhour(int(token[5:7]), int(token[7:9]))
            return Temperature(value, limit, at)
        else:
            return None

    def taf_encode(self):
        return f"T{self.limit}{self.value:02}/{self.at.day:02}{self.at.hour:02}Z"

def decode(s: str):
    return Temperature.taf_decode(s)



import pytest
import detaf
from detaf import wx


def test_integration():
    report = """
    TAF EGAA 081058Z 0812/0912 14010KT 9999 BKN015
    TEMPO 0812/0906 6000 BKN008
    PROB30 TEMPO 0906/0912 BKN008
    """
    actual = detaf.decode(report)
    expected = detaf.TAF(
        icao_identifier="EGAA",
        issue_time=detaf.issue(8, 10, 58),
        weather_conditions=[
            detaf.WeatherCondition(
                period=((8, 12), (9, 12)),
                phenomena=[
                    detaf.Wind(140, 10),
                    detaf.Visibility(9999),
                    detaf.Cloud("BKN", 1500),
                ],
            ),
            detaf.WeatherCondition(
                period=((8, 12), (9, 6)),
                change=detaf.Change.TEMPO,
                phenomena=[detaf.Visibility(6000), detaf.Cloud("BKN", 800)],
            ),
            detaf.WeatherCondition(
                period=((9, 6), (9, 12)),
                probability=30,
                change=detaf.Change.TEMPO,
                phenomena=[detaf.Cloud("BKN", 800)],
            ),
        ],
    )
    assert actual == expected


def test_integration_eigw():
    report = """
    TAF EIDW 081700Z 0818/0918 14010KT 4000 -DZ BKN007
    BECMG 0818/0820 9999 NSW SCT010 BKN015
    BECMG 0901/0903 15005KT
    BECMG 0907/0909 13010KT
    TEMPO 0907/0918 BKN012
    PROB30 TEMPO 0907/0912 4000 -DZ BKN008
    """
    actual = detaf.decode(report)
    expected = detaf.TAF(
        icao_identifier="EIDW",
        issue_time=detaf.issue(8, 17, 0),
        weather_conditions=[
            # 0818/0918 14010KT 4000 -DZ BKN007
            detaf.WeatherCondition(
                period=((8, 18), (9, 18)),
                phenomena=[
                    detaf.Wind(140, 10),
                    detaf.Visibility(4000),
                    wx.Wx(intensity="-", precipitation="DZ"),
                    detaf.Cloud("BKN", 700),
                ],
            ),
            # BECMG 0818/0820 9999 NSW SCT010 BKN015
            detaf.WeatherCondition(
                period=((8, 18), (8, 20)),
                change=detaf.Change.BECMG,
                phenomena=[
                    detaf.Visibility(9999),
                    detaf.NSW.NO_SIGNIFICANT_WEATHER,
                    detaf.Cloud("SCT", 1000),
                    detaf.Cloud("BKN", 1500),
                ],
            ),
            # BECMG 0901/0903 15005KT
            detaf.WeatherCondition(
                period=((9, 1), (9, 3)),
                change=detaf.Change.BECMG,
                phenomena=[
                    detaf.Wind(150, 5),
                ],
            ),
            # BECMG 0907/0909 13010KT
            detaf.WeatherCondition(
                period=((9, 7), (9, 9)),
                change=detaf.Change.BECMG,
                phenomena=[
                    detaf.Wind(130, 10),
                ],
            ),
            # TEMPO 0907/0918 BKN012
            detaf.WeatherCondition(
                period=((9, 7), (9, 18)),
                change=detaf.Change.TEMPO,
                phenomena=[detaf.Cloud("BKN", 1200)],
            ),
            # PROB30 TEMPO 0907/0912 4000 -DZ BKN008
            detaf.WeatherCondition(
                period=((9, 7), (9, 12)),
                change=detaf.Change.TEMPO,
                probability=30,
                phenomena=[
                    detaf.Visibility(4000),
                    wx.Wx(intensity="-", precipitation="DZ"),
                    detaf.Cloud("BKN", 800),
                ],
            ),
        ],
    )
    assert actual == expected


def test_integration_from():
    report = """
    TAF EIDW 081700Z 0818/0918 14010KT 4000 -DZ BKN007
    FM090100 15005KT
    TEMPO 0907/0918 BKN012
    PROB30 TEMPO 0907/0912 4000 -DZ BKN008
    """
    actual = detaf.decode(report).weather_conditions[1]
    expected = detaf.WeatherCondition(
        fm=detaf.From(9, 1, 0), phenomena=[detaf.Wind(direction=150, speed=5)]
    )
    assert actual == expected


@pytest.mark.parametrize(
    "bulletin,expected",
    [
        ("TAF", None),
        ("TAF AMD", detaf.Modification.AMMENDED),
        ("TAF COR", detaf.Modification.CORRECTED),
    ],
)
def test_parse_version(bulletin, expected):
    taf = detaf.decode(bulletin)
    assert isinstance(taf, detaf.TAF)
    assert taf.modification == expected


@pytest.mark.parametrize(
    "bulletin,expected",
    [
        ("TAF", None),
        ("TAF EIDW", "EIDW"),
        ("TAF AMD LFPG", "LFPG"),
    ],
)
def test_parse_icao_code(bulletin, expected):
    taf = detaf.decode(bulletin)
    assert taf.icao_identifier == expected


@pytest.mark.parametrize(
    "bulletin,expected",
    [
        ("TAF LFPG 080500Z", (8, 5, 0)),
    ],
)
def test_parse_issue_time(bulletin, expected):
    taf = detaf.decode(bulletin)
    assert taf.issue_time == expected


@pytest.mark.parametrize(
    "bulletin,expected",
    [
        ("TAF LFPG 080500Z", []),
        ("TAF LFPG 080500Z 0805/0905", [detaf.WeatherCondition(((8, 5), (9, 5)))]),
        (
            "TAF EIDW 080500Z 0805/0905 0807/0809",
            [
                detaf.WeatherCondition(detaf.period((8, 5), (9, 5))),
                detaf.WeatherCondition(detaf.period((8, 7), (8, 9))),
            ],
        ),
        (
            "TAF EIDW 080500Z 0805/0905 PROB30 TEMPO 0807/0809",
            [
                detaf.WeatherCondition(detaf.period((8, 5), (9, 5))),
                detaf.WeatherCondition(
                    detaf.period((8, 7), (8, 9)),
                    probability=30,
                    change=detaf.Change.TEMPO,
                ),
            ],
        ),
        (
            "TAF EIDW 080500Z 0805/0905 TEMPO 0807/0809",
            [
                detaf.WeatherCondition(detaf.period((8, 5), (9, 5))),
                detaf.WeatherCondition(
                    detaf.period((8, 7), (8, 9)), change=detaf.Change.TEMPO
                ),
            ],
        ),
        (
            "TAF EIDW 080500Z 0805/0905 BECMG 0807/0809",
            [
                detaf.WeatherCondition(detaf.period((8, 5), (9, 5))),
                detaf.WeatherCondition(
                    detaf.period((8, 7), (8, 9)), change=detaf.Change.BECMG
                ),
            ],
        ),
    ],
)
def test_parse_weather_conditions(bulletin, expected):
    taf = detaf.decode(bulletin)
    assert taf.weather_conditions == expected


@pytest.mark.parametrize(
    "bulletin,expected",
    [
        ("TAF EIDW 081647Z 0816/0916 9999", [detaf.Visibility(9999)]),
        (
            "TAF EIDW 081647Z 0816/0916 TX27/1815Z",
            [detaf.Temperature(27, "X", at=(18, 15))],
        ),
        (
            "TAF EIDW 081647Z 0816/0916 TN15/1806Z",
            [detaf.Temperature(15, "N", at=(18, 6))],
        ),
    ],
)
def test_parse_visibility(bulletin, expected):
    taf = detaf.decode(bulletin)
    assert taf.weather_conditions[0].phenomena == expected


def test_parse_wx():
    assert wx.decode("-DZ") == wx.Wx(
        intensity=wx.Intensity.LIGHT, precipitation=wx.Precipitation.DRIZZLE
    )


def test_iterable():
    assert list(detaf.decode("TAF COR")) == ["TAF", detaf.Modification.CORRECTED]


def test_variable_wind_conditions():
    text = "VRB02KT"
    assert detaf.Wind.taf_decode(text).direction == "VRB"
    assert detaf.Wind.taf_decode(text).speed == 2
    assert detaf.Wind.taf_decode(text).taf_encode() == text
